# Generated by Django 2.2.24 on 2021-09-24 16:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("common", "0010_remove_teacher_title"),
    ]

    operations = [
        migrations.AddField(
            model_name="student",
            name="login_id",
            field=models.CharField(max_length=64, null=True),
        ),
    ]



# Generated by Django 3.2.20 on 2023-11-06 18:56

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("common", "0046_alter_school_country"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="school",
            name="postcode",
        ),
    ]



# Generated by Django 3.2.13 on 2022-07-13 16:04

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0026_teacher_remove_join_request'),
    ]

    operations = [
        migrations.AddField(
            model_name='class',
            name='created_by',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='created_classes', to='common.teacher'),
        ),
    ]



# Generated by Django 3.2.20 on 2023-09-14 16:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0043_add_total_activity'),
    ]

    operations = [
        migrations.AddField(
            model_name='dailyactivity',
            name='anonymised_unverified_independents',
            field=models.PositiveIntegerField(default=0),
        ),
        migrations.AddField(
            model_name='dailyactivity',
            name='anonymised_unverified_teachers',
            field=models.PositiveIntegerField(default=0),
        ),
        migrations.AddField(
            model_name='totalactivity',
            name='anonymised_unverified_independents',
            field=models.PositiveIntegerField(default=0),
        ),
        migrations.AddField(
            model_name='totalactivity',
            name='anonymised_unverified_teachers',
            field=models.PositiveIntegerField(default=0),
        ),
    ]



# TODO: Move to Address model once we create it
def sanitise_uk_postcode(postcode):
    if len(postcode) >= 5:  # Valid UK postcodes are at least 5 chars long
        outcode = postcode[:-3]  # UK incodes are always 3 characters

        # Insert a space between outcode and incode if there isn't already one
        if not outcode.endswith(" "):
            postcode = postcode[:-3] + " " + postcode[-3:]

    return postcode



# Generated by Django 2.2.24 on 2021-11-09 15:39

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ("common", "0013_class_school"),
    ]

    operations = [
        migrations.AddField(
            model_name="usersession",
            name="login_type",
            field=models.CharField(max_length=100, null=True),
        ),
        migrations.AlterField(
            model_name="class",
            name="creation_time",
            field=models.DateTimeField(default=django.utils.timezone.now, null=True),
        ),
        migrations.AlterField(
            model_name="school",
            name="creation_time",
            field=models.DateTimeField(default=django.utils.timezone.now, null=True),
        ),
    ]



import pgeocode
from django.db import migrations

from ..helpers.organisation import sanitise_uk_postcode


class Migration(migrations.Migration):
    dependencies = [
        ("common", "0040_school_county"),
    ]

    def forwards(apps, schema_editor):
        """Populate the county field for schools in GB"""
        School = apps.get_model("common", "School")
        gb_schools = School.objects.filter(country="GB")
        nomi = pgeocode.Nominatim("GB")

        for school in gb_schools:
            if school.postcode.replace(" ", "") == "":
                school.county = "nan"
                school.save()
            else:
                county = nomi.query_postal_code(sanitise_uk_postcode(school.postcode)).county_name
                school.county = county
                school.save()

    operations = [migrations.RunPython(forwards, reverse_code=migrations.RunPython.noop)]



from builtins import range
from typing import Tuple

from common.helpers.emails import generate_token
from common.helpers.generators import generate_login_id
from common.models import Class, Student
from django.core import mail

from . import email


def generate_school_details():
    name = "Student %d" % generate_school_details.next_id
    password = "Password2"

    generate_school_details.next_id += 1

    return name, password


generate_school_details.next_id = 1


def create_school_student_directly(access_code) -> Tuple[str, str, Student]:
    """Creates a new student in the class with the specified access code.

    Args:
        access_code (str): The access code of the class the created student will be in

    Returns:
        Tuple[str, str, Student]: (name, password, student)
    """
    name, password = generate_school_details()

    klass = Class.objects.get(access_code=access_code)

    student = Student.objects.schoolFactory(klass, name, password)
    return name, password, student


def create_student_with_direct_login(access_code) -> Tuple[Student, str]:
    name, password = generate_school_details()
    klass = Class.objects.get(access_code=access_code)

    # use random string for direct login)
    login_id, hashed_login_id = generate_login_id()
    student = Student.objects.schoolFactory(klass, name, password, hashed_login_id)

    return student, login_id, name, password


def create_independent_student_directly(preverified=True):
    """
    Creates a Student object and makes it independent by generating random details.
    Also verifies the student's email if preverified is True.
    :param preverified: whether or not the independent student's email should be
    verified.
    :return: the student's username, password and the student object itself.
    """
    name, username, email, password = generate_independent_student_details()

    student = Student.objects.independentStudentFactory(name, email, password)

    # verify student
    generate_token(student.new_user, preverified=preverified)

    return username, password, student


def create_school_student(page):
    name, _ = generate_school_details()

    page = page.type_student_name(name).create_students()

    return page, name


def create_many_school_students(page, number_of_students):
    names = ["" for i in range(number_of_students)]

    for i in range(number_of_students):
        names[i], _ = generate_school_details()
        page = page.type_student_name(names[i])

    page = page.create_students()

    return page, names


def generate_independent_student_details():
    name = "Independent Student %d" % generate_independent_student_details.next_id
    email_address = "student%d@codeforlife.com" % generate_independent_student_details.next_id
    username = email_address
    password = "$RFVBGT%^YHNmju7"

    generate_independent_student_details.next_id += 1

    return name, username, email_address, password


generate_independent_student_details.next_id = 1


def signup_duplicate_independent_student_fail(page, duplicate_email=None):
    page = page.go_to_signup_page()

    name, username, email_address, password = generate_independent_student_details()

    if not duplicate_email:
        duplicate_email = email_address

    page = page.independent_student_signup(name, duplicate_email, password=password, confirm_password=password)

    page = page.return_to_home_page()

    page = email.follow_duplicate_account_link_to_login(page, mail.outbox[0], "independent")

    return page, name, username, email_address, password


def create_independent_student(page):
    page = page.go_to_signup_page()

    name, username, email_address, password = generate_independent_student_details()
    page = page.independent_student_signup(name, email_address, password=password, confirm_password=password)

    page = page.return_to_home_page()

    page = email.follow_verify_email_link_to_login(page, mail.outbox[0], "independent")
    mail.outbox = []

    return page, name, username, email_address, password


def verify_email(page):
    assert len(mail.outbox) > 0

    page = email.follow_verify_email_link_to_login(page, mail.outbox[0], "independent")
    mail.outbox = []

    return page



from django.urls import reverse, reverse_lazy


def resetEmailPasswordMessage(request, domain, uid, token, protocol):
    password_reset_uri = reverse_lazy("password_reset_check_and_confirm", kwargs={"uidb64": uid, "token": token})
    url = f"{protocol}://{domain}{password_reset_uri}"
    return {
        "subject": f"Password reset request",
        "message": (
            f"You are receiving this email because you requested "
            f"a password reset for your Code For Life user account.\n\n"
            f"Please go to the following page and choose a new password: {url}"
        ),
    }


def emailVerificationNeededEmail(request, token):
    url = f"{request.build_absolute_uri(reverse('verify_email', kwargs={'token': token}))}"
    privacy_notice_url = f"{request.build_absolute_uri(reverse('privacy_notice'))}"
    terms_url = f"{request.build_absolute_uri(reverse('terms'))}"
    return {
        "subject": f"Email verification ",
        "message": (
            f"Please go to {url} to verify your email address.\n\nBy activating the account you confirm that you have "
            f"read and agreed to our terms ({terms_url}) and our privacy notice ({privacy_notice_url})."
        ),
        "url": {"verify_url": url},
    }


def parentsEmailVerificationNeededEmail(request, user, token):
    url = f"{request.build_absolute_uri(reverse('verify_email', kwargs={'token': token}))}"
    privacy_notice_url = f"{request.build_absolute_uri(reverse('privacy_notice'))}"
    terms_url = f"{request.build_absolute_uri(reverse('terms'))}"
    return {
        "subject": f"Code for Life account request",
        "message": (
            f"{user.first_name} has requested to create a Code for Life account so that they can learn how to code for "
            f"FREE! 🎉\n\n"
            f"{user.first_name} provided your email address as a guardian that is able to read the privacy notice "
            f"documents and agree to the terms and conditions related to our website on their behalf.\n\n"
            f"If you also wish to receive communication from us, you can sign up for newsletters on our website here. 📧\n\n"
            f"Please activate the account for {user.first_name} by following this link: {url}.\n\nBy activating the "
            f"account you confirm that you have read and agreed to our terms ({terms_url}) and our privacy notice "
            f"({privacy_notice_url})."
        ),
        "url": {"verify_url": url},
    }


def emailChangeVerificationEmail(request, token):
    url = f"{request.build_absolute_uri(reverse('verify_email', kwargs={'token': token}))}"
    return {
        "subject": f"Email verification needed",
        "message": (
            f"You are changing your email, please go to "
            f"{url} "
            f"to verify your new email address. If you are not part of Code for Life "
            f"then please ignore this email."
        ),
        "url": {"verify_url": url},
    }


def emailChangeNotificationEmail(request, new_email_address):
    return {
        "subject": f"Email address update",
        "message": (
            f"There is a request to change the email address of your account to "
            f"{new_email_address}. If this was not you, please get in contact with us."
        ),
    }


def userAlreadyRegisteredEmail(request, email, is_independent_student=False):
    if is_independent_student:
        login_url = reverse("independent_student_login")
    else:
        login_url = reverse("teacher_login")

    return {
        "subject": f"Duplicate account",
        "message": (
            f"A user is already registered with this email address: {email}.\n"
            f"If you've already registered, please login: "
            f"{request.build_absolute_uri(login_url)}.\n"
            f"Otherwise please register with a different email address."
        ),
    }


def kickedEmail(request, schoolName):
    return {
        "subject": f"You were successfully released from your school or club",
        "message": (
            f"You have been released from the school or club '{schoolName}'. "
            f"If you think this was an error, please contact the administrator of that "
            f"school or club."
        ),
    }


def adminGivenEmail(request, schoolName):

    url = request.build_absolute_uri(reverse("dashboard"))

    return {
        "subject": f"You have been made a school or club administrator",
        "message": (
            f"Administrator control of the school or club '{schoolName}' has been "
            f"given to you. Go to {url} to start managing your school or club."
        ),
    }


def adminRevokedEmail(request, schoolName):
    return {
        "subject": f"You are no longer a school or club administrator",
        "message": (
            f"Your administrator control of the school or club '{schoolName}' has been "
            f"revoked. If you think this is an error, please contact one of the other "
            f"administrators in your school or club."
        ),
    }


def studentJoinRequestSentEmail(request, schoolName, accessCode):
    return {
        "subject": f"School or club join request sent",
        "message": (
            f"Your request to join the school or club '{schoolName}' in class "
            f"{accessCode} has been sent to that class's teacher, who will either "
            f"accept or deny your request."
        ),
    }


def studentJoinRequestNotifyEmail(request, username, email, accessCode):
    return {
        "subject": f"School or club join request",
        "message": (
            f"There is a request waiting from student with username '{username}' and "
            f"email {email} to join your class {accessCode}. "
            f"Please log in to your dashboard to review the request."
        ),
    }


def studentJoinRequestRejectedEmail(request, schoolName, accessCode):
    return {
        "subject": f"School or club join request rejected",
        "message": (
            f"Your request to join the school or club '{schoolName}' in class "
            f"{accessCode} has been rejected. Speak to your teacher if you think this "
            f"is an error."
        ),
    }


def inviteTeacherEmail(request, schoolName, token, account_exists):
    url = f"{request.build_absolute_uri(reverse('invited_teacher', kwargs={'token': token}))} "

    if account_exists:
        message = (
            f"A teacher at the school '{schoolName}' has invited you to join Code for Life. 🎉 Unfortunately, you "
            f"already have an account with this email address, so you will need to either delete it first or change "
            f"the email registered to your other account. After that, you can complete the registration process, by "
            f"following the link below.\n\n"
            f"{url}"
        )
    else:
        message = (
            f"A teacher at the school '{schoolName}' has invited you to join Code for Life. 🎉 To complete the "
            f"registration process, please create a password by following the link below.\n\n"
            f"{url}"
        )

    return {"subject": f"You've been invited to join Code for Life", "message": message}


def accountDeletionEmail(request):
    return {
        "subject": f"We are sorry to see you go",
        "title": "Your account was successfully deleted",
        "message": (
            f"If you have a moment before you leave us completely, please "
            f"let us know the reason through our super short survey below."
            f"\n\nGive feedback: https://usabi.li/do/d8e0313a31d7/5bef"
            f"\n\nThank you for being part of the Code for Life community!"
        ),
    }



from common.models import DailyActivity, Student, Teacher
from django.test import TestCase
from django.utils import timezone

from ..helpers.organisation import sanitise_uk_postcode
from .utils.classes import create_class_directly
from .utils.organisation import create_organisation_directly, join_teacher_to_organisation
from .utils.student import create_independent_student_directly
from .utils.teacher import signup_teacher_directly


class TestModels(TestCase):
    def test_indep_student_pending_class_request_on_delete(self):
        """
        Given a class and an independent student,
        When the student makes a request to join the class and the class is deleted,
        Then the student's pending class request field is set to null.
        """
        teacher_email, _ = signup_teacher_directly()
        create_organisation_directly(teacher_email)
        class_name = "Test Class"
        klass, _, _ = create_class_directly(teacher_email, class_name)

        username, _, indep_student = create_independent_student_directly()

        assert indep_student.is_independent()
        assert indep_student.pending_class_request is None

        indep_student.pending_class_request = klass
        indep_student.save()

        klass.anonymise()

        indep_student = Student.objects.get(new_user__username=username)

        assert indep_student.pending_class_request is None

    def test_creation_time(self):
        teacher_email, _ = signup_teacher_directly()

        sometime = timezone.now()  # mark time before the school creation
        school = create_organisation_directly(teacher_email)

        # check the creation time
        assert school.creation_time > sometime

        sometime = timezone.now()  # mark time before the class creation
        klass, name, access_code = create_class_directly(teacher_email)
        # check the creation time
        assert klass.creation_time > sometime

    def test_school_admins(self):
        """
        Test that only the admins of a school are returned by the school.admins() function.
        """
        email1, password1 = signup_teacher_directly()
        email2, password2 = signup_teacher_directly()
        email3, password3 = signup_teacher_directly()
        school = create_organisation_directly(email1)
        join_teacher_to_organisation(email2, school.name)
        join_teacher_to_organisation(email3, school.name, is_admin=True)

        teacher1 = Teacher.objects.get(new_user__username=email1)
        teacher2 = Teacher.objects.get(new_user__username=email2)
        teacher3 = Teacher.objects.get(new_user__username=email3)

        assert len(school.admins()) == 2
        assert teacher1 in school.admins()
        assert teacher2 not in school.admins()
        assert teacher3 in school.admins()

    def test_sanitise_uk_postcode(self):
        postcode_with_space = "AL10 9NE"
        postcode_without_space = "AL109UL"
        invalid_postcode = "123"

        assert sanitise_uk_postcode(postcode_with_space) == "AL10 9NE"  # Check it stays the same
        assert sanitise_uk_postcode(postcode_without_space) == "AL10 9UL"  # Check a space is added
        assert sanitise_uk_postcode(invalid_postcode) == "123"  # Check nothing happens

    def test_daily_activity_serializer(self):
        daily_activity = DailyActivity()

        assert (
            str(daily_activity)
            == f"Activity on {daily_activity.date}: CSV clicks: 0, login cards clicks: 0, primary pack downloads: 0, python pack downloads: 0, level control submits: 0, teacher lockout resets: 0, indy lockout resets: 0, school student lockout resets: 0, unverified teachers anonymised: 0, unverified independents anonymised: 0"
        )



# Generated by Django 3.2.20 on 2023-08-04 14:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0039_copy_email_to_username'),
    ]

    operations = [
        migrations.AddField(
            model_name='school',
            name='county',
            field=models.CharField(blank=True, max_length=50, null=True),
        ),
    ]



import pytest
from django_test_migrations.migrator import Migrator


@pytest.mark.django_db
def test_0048_unique_school_names(migrator: Migrator):
    state = migrator.apply_initial_migration(("common", "0047_delete_school_postcode"))
    School = state.apps.get_model("common", "School")

    school_name = "ExampleSchool"
    School.objects.bulk_create(
        [
            School(name=school_name),
            School(name=school_name),
            School(name=f"{school_name} 1"),
        ]
    )
    school_ids = list(School.objects.order_by("-id")[:3].values_list("id", flat=True))
    school_ids.reverse()

    migrator.apply_tested_migration(("common", "0048_unique_school_names"))
    School = state.apps.get_model("common", "School")

    def assert_school_name(index: int, name: str):
        assert School.objects.get(id=school_ids[index]).name == name

    assert_school_name(0, school_name)
    assert_school_name(1, f"{school_name} 2")
    assert_school_name(2, f"{school_name} 1")



# Generated by Django 3.2.20 on 2023-11-06 18:56

from django.apps.registry import Apps
from django.db import migrations, models


def unique_school_names(apps: Apps, *args):
    School = apps.get_model("common", "School")

    def get_school_name(school_name: str, school_name_number: int):
        return f"{school_name} {school_name_number}"

    school_values = School.objects.values("name").annotate(name_count=models.Count("name")).filter(name_count__gt=1)
    for school_value in school_values:
        school_name = school_value["name"]
        school_name_number = 1

        schools = list(School.objects.filter(name=school_name).order_by("id"))
        for index in range(1, len(schools)):
            school = schools[index]

            school.name = get_school_name(school_name, school_name_number)
            while School.objects.filter(name=school.name).exists():
                school_name_number += 1
                school.name = get_school_name(school_name, school_name_number)

            school.save()


class Migration(migrations.Migration):
    dependencies = [
        ("common", "0047_delete_school_postcode"),
    ]

    operations = [
        migrations.RunPython(code=unique_school_names, reverse_code=migrations.RunPython.noop),
        migrations.AlterField(
            model_name="school",
            name="name",
            field=models.CharField(max_length=200, unique=True),
        ),
    ]



# Generated by Django 3.2.18 on 2023-04-13 18:41

from django.db import migrations
from django.db.models import F


def copy_email_to_username(apps, schema):
    User = apps.get_model("auth", "User")
    User.objects.exclude(email="").exclude(email__isnull=True).exclude(email=F("username")).update(username=F("email"))


class Migration(migrations.Migration):

    dependencies = [
        ("common", "0038_delete_emailverification"),
    ]

    operations = [
        migrations.RunPython(
            code=copy_email_to_username, reverse_code=migrations.RunPython.noop
        )
    ]




from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("common", "0036_rename_awaiting_email_verification_userprofile_is_verified"),
    ]

    def forwards(apps, schema_editor):
        """ Finds the users of verified Email Verification objects and sets their `is_verified` to True """
        UserProfile = apps.get_model("common", "UserProfile")
        db_alias = schema_editor.connection.alias
        UserProfile.objects.using(db_alias).filter(user__email_verifications__verified=True).update(is_verified=True)

    def backwards(apps, schema_editor):
        """ Finds the users of verified Email Verification objects and sets their `is_verified` to False """
        UserProfile = apps.get_model("common", "UserProfile")
        db_alias = schema_editor.connection.alias
        UserProfile.objects.using(db_alias).filter(user__email_verifications__verified=True).update(is_verified=False)

    operations = [migrations.RunPython(forwards, reverse_code=backwards)]



# Generated by Django 3.2.18 on 2023-04-13 18:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0037_migrate_email_verification'),
    ]

    operations = [
        migrations.DeleteModel(
            name='EmailVerification',
        ),
    ]



from common import app_settings


def module_name(request):
    return {"module_name": app_settings.MODULE_NAME}


def cookie_management_enabled(request):
    return {"cookie_management_enabled": app_settings.COOKIE_MANAGEMENT_ENABLED}



# Generated by Django 2.2.24 on 2021-10-15 15:10

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("common", "0012_usersession"),
    ]

    operations = [
        migrations.AddField(
            model_name="class",
            name="creation_time",
            field=models.DateTimeField(null=True),
        ),
        migrations.AddField(
            model_name="school",
            name="creation_time",
            field=models.DateTimeField(null=True),
        ),
        migrations.AddField(
            model_name="usersession",
            name="class_field",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="common.Class",
            ),
        ),
        migrations.AddField(
            model_name="usersession",
            name="school",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="common.School",
            ),
        ),
    ]



# Generated by Django 3.2.20 on 2023-09-07 01:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0041_populate_gb_counties'),
    ]

    operations = [
        migrations.CreateModel(
            name='TotalActivity',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('teacher_registrations', models.PositiveIntegerField(default=0)),
                ('student_registrations', models.PositiveIntegerField(default=0)),
                ('independent_registrations', models.PositiveIntegerField(default=0)),
            ],
            options={
                'verbose_name_plural': 'Total activity',
            },
        ),
    ]



# Generated by Django 3.2.13 on 2022-05-17 17:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0022_school_cleanup'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='school',
            options={},
        ),
        migrations.AddField(
            model_name='userprofile',
            name='aimmo_badges',
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
    ]



import re
from builtins import str


def follow_verify_email_link_to_onboarding(page, email):
    _follow_verify_email_link(page, email)

    return go_to_teacher_login_page(page.browser)


def follow_verify_email_link_to_teacher_dashboard(page, email):
    _follow_verify_email_link(page, email)

    return go_to_teacher_dashboard_page(page.browser)


def follow_verify_email_link_to_login(page, email, user_type):
    _follow_verify_email_link(page, email)

    if user_type == "teacher":
        return go_to_teacher_login_page(page.browser)
    elif user_type == "independent":
        return go_to_independent_student_login_page(page.browser)


def follow_duplicate_account_link_to_login(page, email, user_type):
    _follow_duplicate_account_email_link(page, email)

    if user_type == "teacher":
        return go_to_teacher_login_page(page.browser)
    elif user_type == "independent":
        return go_to_independent_student_login_page(page.browser)


def _follow_verify_email_link(page, email):
    message = str(email.message())
    prefix = '<p>Please go to <a href="'
    i = str.find(message, prefix) + len(prefix)
    suffix = '" rel="nofollow">'
    j = str.find(message, suffix, i)
    page.browser.get(message[i:j])


def _follow_duplicate_account_email_link(page, email):
    message = str(email.message())
    prefix = 'please login: <a href="'
    i = str.find(message, prefix) + len(prefix)
    suffix = '" rel="nofollow">'
    j = str.find(message, suffix, i)
    page.browser.get(message[i:j])


def follow_reset_email_link(browser, email):
    message = str(email.body)
    link = re.search("http.+/", message).group(0)[:-1]
    browser.get(link)

    from portal.tests.pageObjects.portal.password_reset_form_page import PasswordResetPage

    return PasswordResetPage(browser)


def follow_change_email_link_to_dashboard(page, email):
    _follow_change_email_link(page, email)

    return go_to_teacher_login_page(page.browser)


def follow_change_email_link_to_independent_dashboard(page, email):
    _follow_change_email_link(page, email)

    return go_to_independent_student_login_page(page.browser)


def _follow_change_email_link(page, email):
    message = str(email.message())
    prefix = "please go to "
    i = str.find(message, prefix) + len(prefix)
    suffix = " to verify"
    j = str.find(message, suffix, i)
    page.browser.get(message[i:j])


def go_to_teacher_login_page(browser):
    from portal.tests.pageObjects.portal.teacher_login_page import TeacherLoginPage

    return TeacherLoginPage(browser)


def go_to_teacher_dashboard_page(browser):
    from portal.tests.pageObjects.portal.teach.dashboard_page import TeachDashboardPage

    return TeachDashboardPage(browser)


def go_to_independent_student_login_page(browser):
    from portal.tests.pageObjects.portal.independent_login_page import IndependentStudentLoginPage

    return IndependentStudentLoginPage(browser)



from django.contrib.auth.models import User
from django.utils import timezone


def get_superuser():
    """Get a superuser for testing, or create one if there isn't one."""
    try:
        return User.objects.get(username="superuser")
    except User.DoesNotExist:
        return User.objects.create_superuser("superuser", "superuser@codeforlife.education", "password")


def create_user_directly(active=True, **kwargs):
    """Create a user in the database."""
    days_to_subtract = 10 if active else 2000
    username = "old_user+{:d}".format(create_user_directly.next_id)
    user = User.objects.create_user(username, password="password")
    user.last_login = timezone.now() - timezone.timedelta(days=days_to_subtract)
    user.date_joined = timezone.now() - timezone.timedelta(days=days_to_subtract - 1)
    user.save()

    create_user_directly.next_id += 1

    return user


create_user_directly.next_id = 1



from functools import wraps

from common.utils import using_two_factor
from django.http import Http404
from django.http import HttpResponseRedirect
from django.urls import reverse_lazy
from rest_framework import permissions


def has_completed_auth_setup(user):
    return (not using_two_factor(user)) or (user.userprofile.is_verified and using_two_factor(user))


class LoggedInAsTeacher(permissions.BasePermission):
    def has_permission(self, request, view):
        return logged_in_as_teacher(request.user)


def logged_in_as_teacher(user):
    try:
        return user.userprofile.teacher and has_completed_auth_setup(user)
    except AttributeError:
        return False


def logged_in_as_student(u):
    try:
        if u.userprofile.student:
            return True
    except AttributeError:
        return False


def logged_in_as_independent_student(u):
    return logged_in_as_student(u) and u.userprofile.student.is_independent()


def logged_in_as_school_student(u):
    return logged_in_as_student(u) and not u.userprofile.student.is_independent()


def check_teacher_authorised(request, class_teacher):
    current_teacher_owns_the_class = class_teacher == request.user.new_teacher
    is_current_teacher_school_admin = (
        class_teacher.school == request.user.new_teacher.school and request.user.new_teacher.is_admin
    )

    if not (current_teacher_owns_the_class or is_current_teacher_school_admin):
        raise Http404


def not_logged_in(u):
    try:
        if u.userprofile:
            return False
    except AttributeError:
        return True


def not_fully_logged_in(u):
    return not_logged_in(u) or (not logged_in_as_student(u) and not logged_in_as_teacher(u))


def teacher_verified(view_func):
    @wraps(view_func)
    def wrapped(request, *args, **kwargs):
        u = request.user
        try:
            if not u.userprofile.teacher or not has_completed_auth_setup(u):
                return HttpResponseRedirect(reverse_lazy("teach"))
        except AttributeError:
            return HttpResponseRedirect(reverse_lazy("teach"))
        return view_func(request, *args, **kwargs)

    return wrapped


class CanDeleteGame(permissions.BasePermission):
    def has_permission(self, request, view):
        u = request.user
        try:
            return u.userprofile.teacher and has_completed_auth_setup(u)
        except AttributeError:
            return False



# Generated by Django 3.2.16 on 2023-01-27 03:17

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('common', '0034_dailyactivity_daily_school_student_lockout_reset'),
    ]

    operations = [
        migrations.RenameField(
            model_name='dailyactivity',
            old_name='daily_indy_lockout_reset',
            new_name='indy_lockout_resets',
        ),
        migrations.RenameField(
            model_name='dailyactivity',
            old_name='daily_school_student_lockout_reset',
            new_name='school_student_lockout_resets',
        ),
        migrations.RenameField(
            model_name='dailyactivity',
            old_name='daily_teacher_lockout_reset',
            new_name='teacher_lockout_resets',
        ),
    ]



# Generated by Django 3.2.16 on 2022-12-09 13:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0031_improve_admin_panel'),
    ]

    operations = [
        migrations.AddField(
            model_name='dailyactivity',
            name='level_control_submits',
            field=models.PositiveBigIntegerField(default=0),
        ),
    ]



# Generated by Django 3.2.18 on 2023-04-11 14:31

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("common", "0035_rename_lockout_fields"),
    ]

    operations = [
        migrations.RenameField(
            model_name="userprofile",
            old_name="awaiting_email_verification",
            new_name="is_verified",
        ),
    ]



# Generated by Django 3.2.20 on 2023-09-27 13:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0044_update_activity_models'),
    ]

    operations = [
        migrations.AddField(
            model_name='userprofile',
            name='last_otp_for_time',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='userprofile',
            name='otp_secret',
            field=models.CharField(blank=True, max_length=40, null=True),
        ),
    ]



# Generated by Django 3.2.13 on 2022-04-29 17:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0020_class_is_active_and_null_access_code'),
    ]

    operations = [
        migrations.AddField(
            model_name='school',
            name='is_active',
            field=models.BooleanField(default=True),
        ),
        migrations.AlterField(
            model_name='school',
            name='postcode',
            field=models.CharField(max_length=10, null=True),
        ),
        migrations.AlterField(
            model_name='school',
            name='town',
            field=models.CharField(max_length=200, null=True),
        ),
    ]



# Generated by Django 3.2.13 on 2022-06-15 10:22

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0024_teacher_invited_by'),
    ]

    operations = [
        migrations.CreateModel(
            name='SchoolTeacherInvitation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('token', models.CharField(max_length=32)),
                ('invited_teacher_first_name', models.CharField(max_length=150)),
                ('invited_teacher_last_name', models.CharField(max_length=150)),
                ('invited_teacher_email', models.EmailField(max_length=254)),
                ('invited_teacher_is_admin', models.BooleanField(default=False)),
                ('expiry', models.DateTimeField()),
                ('creation_time', models.DateTimeField(default=django.utils.timezone.now, null=True)),
                ('is_active', models.BooleanField(default=True)),
                ('from_teacher', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='school_invitations', to='common.teacher')),
                ('school', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='teacher_invitations', to='common.school')),
            ],
        ),
    ]



from django.db import migrations


def add_total_activity(apps, schema_editor):
    """
    This creates the only TotalActivity entry that we need to record total activity.
    Initialises it with the total number of registrations at the time of the migration.
    """
    TotalActivity = apps.get_model("common", "TotalActivity")
    Teacher = apps.get_model("common", "Teacher")
    Student = apps.get_model("common", "Student")
    TotalActivity.objects.create(
        teacher_registrations=Teacher.objects.all().count(),
        student_registrations=Student.objects.filter(class_field__isnull=False).count(),
        independent_registrations=Student.objects.filter(class_field__isnull=True).count(),
    )


def remove_total_activity(apps, schema_editor):
    TotalActivity = apps.get_model("common", "TotalActivity")
    TotalActivity.objects.get(id=1).delete()


class Migration(migrations.Migration):

    dependencies = [
        ("common", "0042_totalactivity"),
    ]

    operations = [migrations.RunPython(add_total_activity, remove_total_activity)]



# Generated by Django 3.2.20 on 2023-11-06 16:04

from django.db import migrations
import django_countries.fields


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0045_otp'),
    ]

    operations = [
        migrations.AlterField(
            model_name='school',
            name='country',
            field=django_countries.fields.CountryField(blank=True, max_length=2, null=True),
        ),
    ]



"""CSP Config"""

from .app_settings import domain, MODULE_NAME

CSP_DEFAULT_SRC = ("self",)
CSP_CONNECT_SRC = (
    "'self'",
    "https://*.onetrust.com/",
    "https://api.pwnedpasswords.com",
    "https://euc-widget.freshworks.com/",
    "https://codeforlife.freshdesk.com/",
    "https://api.iconify.design/",
    "https://api.simplesvg.com/",
    "https://api.unisvg.com/",
    "https://www.google-analytics.com/",
    "https://pyodide-cdn2.iodide.io/v0.15.0/full/",
    "https://crowdin.com/",
    f"wss://{MODULE_NAME}-aimmo.codeforlife.education/",
    f"https://{MODULE_NAME}-aimmo.codeforlife.education/",
)
CSP_FONT_SRC = ("'self'", "https://fonts.gstatic.com/", "https://fonts.googleapis.com/", "https://use.typekit.net/")
CSP_SCRIPT_SRC = (
    "'self'",
    "'unsafe-inline'",
    "'unsafe-eval'",
    "https://cdnjs.cloudflare.com/ajax/libs/crypto-js/4.0.0/crypto-js.min.js",
    "https://cdn.crowdin.com/",
    "https://*.onetrust.com/",
    "https://code.jquery.com/",
    "https://euc-widget.freshworks.com/",
    "https://cdn-ukwest.onetrust.com/",
    "https://code.iconify.design/2/2.0.3/iconify.min.js",
    "https://www.googletagmanager.com/",
    "https://cdn.mouseflow.com/",
    "https://www.recaptcha.net/",
    "https://www.google.com/recaptcha/",
    "https://www.gstatic.com/recaptcha/",
    "https://use.typekit.net/mrl4ieu.js",
    "https://pyodide-cdn2.iodide.io/v0.15.0/full/",
    f"{domain()}/static/portal/",
    f"{domain()}/static/common/",
)
CSP_STYLE_SRC = (
    "'self'",
    "'unsafe-inline'",
    "https://euc-widget.freshworks.com/",
    "https://cdn-ukwest.onetrust.com/",
    "https://fonts.googleapis.com/",
    "https://code.jquery.com/ui/1.13.1/themes/base/jquery-ui.css",
    "https://cdn.crowdin.com/",
    f"{domain()}/static/portal/",
)
CSP_FRAME_SRC = (
    "https://storage.googleapis.com/",
    "https://www.youtube-nocookie.com/",
    "https://www.recaptcha.net/",
    "https://www.google.com/recaptcha/",
    "https://crowdin.com/",
    f"{domain()}/static/common/img/",
    f"{domain()}/static/game/image/",
)
CSP_IMG_SRC = (
    "'self'",
    "https://storage.googleapis.com/codeforlife-assets/images/",
    "https://cdn-ukwest.onetrust.com/",
    "https://p.typekit.net/",
    "https://cdn.crowdin.com/",
    "https://crowdin-static.downloads.crowdin.com/",
    "data:",
    f"{domain()}/static/portal/img/",
    f"{domain()}/static/portal/static/portal/img/",
    f"{domain()}/static/portal/img/",
    f"{domain()}/favicon.ico",
    f"{domain()}/img/",
    f"{domain()}/account/two_factor/qrcode/",
    f"{domain()}/static/",
    f"{domain()}/static/game/image/",
    f"{domain()}/static/game/raphael_image/",
    f"{domain()}/static/game/js/blockly/media/",
    f"{domain()}/static/icons/",
)
CSP_OBJECT_SRC = (f"{domain()}/static/common/img/", f"{domain()}/static/game/image/")
CSP_MEDIA_SRC = (f"{domain()}/static/game/sound/", f"{domain()}/static/game/js/blockly/media/")



from typing import Tuple

from common.helpers.generators import generate_access_code
from common.models import Class, Teacher


def generate_details():
    name = "Class %d" % generate_details.next_id
    access_code = generate_access_code()

    generate_details.next_id += 1

    return name, access_code


generate_details.next_id = 1


def create_class_directly(teacher_email: str, class_name: str = None) -> Tuple[Class, str, str]:
    """Generate a class with the details given.

    Args:
        teacher_email (str): The email of the teacher that will own the class.
        class_name (str, optional): The name of the class. Defaults to auto-generated name.

    Returns:
        (class: Class, name: str, access_code: str): A tuple with the class model instance, name and access code.
    """
    name, access_code = generate_details()

    if class_name is not None:
        name = class_name

    teacher = Teacher.objects.get(new_user__email=teacher_email)

    klass = Class.objects.create(name=name, access_code=access_code, teacher=teacher)

    return klass, name, access_code



from common.models import School, Teacher


def generate_details(**kwargs):
    name = kwargs.get("name", "School %d" % generate_details.next_id)

    generate_details.next_id += 1

    return name


generate_details.next_id = 1


def create_organisation_directly(teacher_email, **kwargs):
    name = generate_details(**kwargs)

    school = School.objects.create(name=name, country="GB")

    teacher = Teacher.objects.get(new_user__email=teacher_email)
    teacher.school = school
    teacher.is_admin = True
    teacher.save()

    return school


def join_teacher_to_organisation(teacher_email, org_name, is_admin=False):
    teacher = Teacher.objects.get(new_user__email=teacher_email)
    school = School.objects.get(name=org_name)

    teacher.school = school
    teacher.is_admin = is_admin
    teacher.save()


def create_organisation(page, password):
    name = generate_details()
    page = page.create_organisation(name, password)

    return page, name



# Generated by Django 3.2.16 on 2023-01-24 15:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0033_password_reset_tracking_fields'),
    ]

    operations = [
        migrations.AddField(
            model_name='dailyactivity',
            name='daily_school_student_lockout_reset',
            field=models.PositiveIntegerField(default=0),
        ),
    ]



# Generated by Django 3.2.16 on 2022-11-22 16:42

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0030_add_maintenance_banner'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='dailyactivity',
            options={'verbose_name_plural': 'Daily activities'},
        ),
        migrations.AlterField(
            model_name='student',
            name='blocked_time',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='student',
            name='class_field',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='students', to='common.class'),
        ),
        migrations.AlterField(
            model_name='student',
            name='pending_class_request',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='class_request', to='common.class'),
        ),
        migrations.AlterField(
            model_name='teacher',
            name='school',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='teacher_school', to='common.school'),
        ),
    ]



from django.db import migrations


def copy_email_to_username(apps, schema):
    Student = apps.get_model("common", "Student")
    independent_students = Student.objects.filter(
        class_field__isnull=True, new_user__is_active=True
    )
    for student in independent_students:
        student.new_user.username = student.new_user.email
        student.new_user.save()


class Migration(migrations.Migration):

    dependencies = [
        ("common", "0016_joinreleasestudent"),
    ]

    operations = [
        migrations.RunPython(
            code=copy_email_to_username, reverse_code=migrations.RunPython.noop
        )
    ]



import datetime
import json
import re
from enum import Enum, auto
from uuid import uuid4

import jwt
from common import app_settings
from common.app_settings import domain
from common.email_messages import (
    emailChangeNotificationEmail,
    emailChangeVerificationEmail,
    emailVerificationNeededEmail,
    parentsEmailVerificationNeededEmail,
)
from common.models import Teacher, Student
from django.conf import settings
from django.contrib.auth.models import User
from django.core.mail import EmailMultiAlternatives
from django.http import HttpResponse
from django.template import loader
from django.utils import timezone
from requests import post, get, put, delete
from requests.exceptions import RequestException

NOTIFICATION_EMAIL = "Code For Life Notification <" + app_settings.EMAIL_ADDRESS + ">"
VERIFICATION_EMAIL = "Code For Life Verification <" + app_settings.EMAIL_ADDRESS + ">"
PASSWORD_RESET_EMAIL = "Code For Life Password Reset <" + app_settings.EMAIL_ADDRESS + ">"
INVITE_FROM = "Code For Life Invitation <" + app_settings.EMAIL_ADDRESS + ">"


class DotmailerUserType(Enum):
    TEACHER = auto()
    STUDENT = auto()
    NO_ACCOUNT = auto()


def send_email(
    sender,
    recipients,
    subject,
    text_content,
    title,
    replace_url=None,
    plaintext_template="email.txt",
    html_template="email.html",
):
    # add in template for templates to message

    # setup templates
    plaintext = loader.get_template(plaintext_template)
    html = loader.get_template(html_template)
    plaintext_email_context = {"content": text_content}
    html_email_context = {"content": text_content, "title": title, "url_prefix": domain()}

    # render templates
    plaintext_body = plaintext.render(plaintext_email_context)
    original_html_body = html.render(html_email_context)
    html_body = original_html_body

    if replace_url:
        verify_url = replace_url["verify_url"]
        verify_replace_url = re.sub(f"(.*/verify_email/)(.*)", f"\\1", verify_url)
        html_body = re.sub(f"({verify_url})(.*){verify_url}", f"\\1\\2{verify_replace_url}", original_html_body)

    # make message using templates
    message = EmailMultiAlternatives(subject, plaintext_body, sender, recipients)
    message.attach_alternative(html_body, "text/html")

    message.send()


def generate_token(user, new_email="", preverified=False):
    if preverified:
        user.userprofile.is_verified = preverified
        user.userprofile.save()

    return generate_token_for_email(user.email, new_email)


def generate_token_for_email(email: str, new_email: str = ""):
    return jwt.encode(
        {
            "email": email,
            "new_email": new_email,
            "email_verification_token": uuid4().hex[:30],
            "expires": (timezone.now() + datetime.timedelta(hours=1)).timestamp(),
        },
        settings.SECRET_KEY,
        algorithm="HS256",
    )


def _newsletter_ticked(data):
    return "newsletter_ticked" in data and data["newsletter_ticked"]


def send_verification_email(request, user, data, new_email=None, age=None):
    """
    Sends emails relating to email address verification.

    On registration:
    - if the user is under 13, send a verification email addressed to the parent / guardian
    - if the user is over 13, send a regular verification email
    - if the user is a student who has requested to sign up to the newsletter, handle their Dotmailer subscription

    On email address update:
    - sends an email to the old address alerting the user that an email change request has occurred
    - sends an email to the new address requesting email address verification

    :param request: The Django form request
    :param user: The user object concerned with the email verification
    :param data: The data inputted by the user in the Django form.
    :param new_email: New email the user wants to associate to their account - if not provided, it means the user is
    registering a new account
    :param age: The user's age (collected only for the purposes of this function and if the user is an independent
    student)
    """

    # verifying first email address (registration)
    if not new_email:
        verification = generate_token(user)

        # if the user is a teacher
        if age is None:
            message = emailVerificationNeededEmail(request, verification)
            send_email(
                VERIFICATION_EMAIL,
                [user.email],
                message["subject"],
                message["message"],
                message["subject"],
                replace_url=message["url"],
            )

            if _newsletter_ticked(data):
                add_to_dotmailer(user.first_name, user.last_name, user.email, DotmailerUserType.TEACHER)
        # if the user is an independent student
        else:
            if age < 13:
                message = parentsEmailVerificationNeededEmail(request, user, verification)
                send_email(
                    VERIFICATION_EMAIL,
                    [user.email],
                    message["subject"],
                    message["message"],
                    message["subject"],
                    replace_url=message["url"],
                )
            else:
                message = emailVerificationNeededEmail(request, verification)
                send_email(
                    VERIFICATION_EMAIL,
                    [user.email],
                    message["subject"],
                    message["message"],
                    message["subject"],
                    replace_url=message["url"],
                )

            if _newsletter_ticked(data):
                add_to_dotmailer(user.first_name, user.last_name, user.email, DotmailerUserType.STUDENT)
    # verifying change of email address.
    else:
        verification = generate_token(user, new_email)

        message = emailChangeVerificationEmail(request, verification)
        send_email(
            VERIFICATION_EMAIL,
            [new_email],
            message["subject"],
            message["message"],
            message["subject"],
            replace_url=message["url"],
        )


def add_to_dotmailer(first_name: str, last_name: str, email: str, user_type: DotmailerUserType):
    try:
        create_contact(first_name, last_name, email)
        add_contact_to_address_book(first_name, last_name, email, user_type)
    except RequestException:
        return HttpResponse(status=404)


def create_contact(first_name, last_name, email):
    url = app_settings.DOTMAILER_CREATE_CONTACT_URL
    body = {
        "contact": {
            "email": email,
            "optInType": "VerifiedDouble",
            "emailType": "Html",
            "dataFields": [
                {"key": "FIRSTNAME", "value": first_name},
                {"key": "LASTNAME", "value": last_name},
                {"key": "FULLNAME", "value": f"{first_name} {last_name}"},
            ],
        },
        "consentFields": [{"fields": [{"key": "DATETIMECONSENTED", "value": datetime.datetime.now().__str__()}]}],
        "preferences": app_settings.DOTMAILER_DEFAULT_PREFERENCES,
    }

    post(url, json=body, auth=(app_settings.DOTMAILER_USER, app_settings.DOTMAILER_PASSWORD))


def add_contact_to_address_book(first_name: str, last_name: str, email: str, user_type: DotmailerUserType):
    main_address_book_url = app_settings.DOTMAILER_MAIN_ADDRESS_BOOK_URL

    body = {
        "email": email,
        "optInType": "VerifiedDouble",
        "emailType": "Html",
        "dataFields": [
            {"key": "FIRSTNAME", "value": first_name},
            {"key": "LASTNAME", "value": last_name},
            {"key": "FULLNAME", "value": f"{first_name} {last_name}"},
        ],
    }

    post(main_address_book_url, json=body, auth=(app_settings.DOTMAILER_USER, app_settings.DOTMAILER_PASSWORD))

    specific_address_book_url = app_settings.DOTMAILER_NO_ACCOUNT_ADDRESS_BOOK_URL

    if user_type == DotmailerUserType.TEACHER:
        specific_address_book_url = app_settings.DOTMAILER_TEACHER_ADDRESS_BOOK_URL
    elif user_type == DotmailerUserType.STUDENT:
        specific_address_book_url = app_settings.DOTMAILER_STUDENT_ADDRESS_BOOK_URL

    post(specific_address_book_url, json=body, auth=(app_settings.DOTMAILER_USER, app_settings.DOTMAILER_PASSWORD))


def delete_contact(email: str):
    try:
        user = get_dotmailer_user_by_email(email)
        user_id = user.get("id")
        if user_id:
            url = app_settings.DOTMAILER_DELETE_USER_BY_ID_URL.replace("ID", str(user_id))
            delete(url, auth=(app_settings.DOTMAILER_USER, app_settings.DOTMAILER_PASSWORD))
    except RequestException:
        return HttpResponse(status=404)


def get_dotmailer_user_by_email(email):
    url = app_settings.DOTMAILER_GET_USER_BY_EMAIL_URL.replace("EMAIL", email)

    response = get(url, auth=(app_settings.DOTMAILER_USER, app_settings.DOTMAILER_PASSWORD))

    return json.loads(response.content)


def add_consent_record_to_dotmailer_user(user):
    consent_date_time = datetime.datetime.now().__str__()

    url = app_settings.DOTMAILER_PUT_CONSENT_DATA_URL.replace("USER_ID", str(user["id"]))
    body = {
        "contact": {
            "email": user["email"],
            "optInType": user["optInType"],
            "emailType": user["emailType"],
            "dataFields": user["dataFields"],
        },
        "consentFields": [{"fields": [{"key": "DATETIMECONSENTED", "value": consent_date_time}]}],
    }

    put(url, json=body, auth=(app_settings.DOTMAILER_USER, app_settings.DOTMAILER_PASSWORD))


def send_dotmailer_consent_confirmation_email_to_user(user):
    url = app_settings.DOTMAILER_SEND_CAMPAIGN_URL
    campaign_id = app_settings.DOTMAILER_THANKS_FOR_STAYING_CAMPAIGN_ID
    body = {"campaignID": campaign_id, "contactIds": [str(user["id"])]}

    post(url, json=body, auth=(app_settings.DOTMAILER_USER, app_settings.DOTMAILER_PASSWORD))


def update_indy_email(user, request, data):
    changing_email = False
    new_email = data["email"]

    if new_email != "" and new_email != user.email:
        changing_email = True
        users_with_email = User.objects.filter(email=new_email)

        message = emailChangeNotificationEmail(request, new_email)
        send_email(VERIFICATION_EMAIL, [user.email], message["subject"], message["message"], message["subject"])

        # email is available
        if not users_with_email.exists():
            # new email to set and verify
            send_verification_email(request, user, data, new_email)
    return changing_email, new_email


def update_email(user: Teacher or Student, request, data):
    changing_email = False
    new_email = data["email"]

    if new_email != "" and new_email != user.new_user.email:
        changing_email = True
        users_with_email = User.objects.filter(email=new_email)

        message = emailChangeNotificationEmail(request, new_email)
        send_email(
            VERIFICATION_EMAIL, [user.new_user.email], message["subject"], message["message"], message["subject"]
        )

        # email is available
        if not users_with_email.exists():
            # new email to set and verify
            send_verification_email(request, user.new_user, data, new_email)
    return changing_email, new_email
